% "Example on Sensitivity Enhancement in Closed Loop".
% We consider a 4-story shear building with 2% damping. Sensors measure displacements at all coordinates 
% and we have two acctuators located in levels 1 and 2. The slope of the actuators is such that the 
% horizontal component of the force is 0.9. We do not optimize but show how the gain is computed 
% for one specific eigenstructure target where the first two poles are shifted towards the
% origin by 10% and the constants that collapse the eigenvector bases are taken as [1;1];
% ***********************
% ***********************
% ***********************
clear
close all
% System
k=500;
m4=1.5;
dof=4;
sen=[1 2 3 4];
K=[2 -1 0 0;-1 2 -1 0;0 -1 2 -1;0 0 -1 1]*k;
M=eye(dof)*m4;
damp=0.02*ones(dof,1);
Cdam=classical_damp(K,M,damp);
% Compute the system poles
Ac=[zeros(dof) eye(dof);-inv(M)*K -inv(M)*Cdam];
lamOL=eig(Ac);
% Sort
lamOL=esort(lamOL);
% Select the closed loop pole possitions - Since we have 4 sensors we can
% only place (exactly) 4 poles - which we take as the first and the second
% plus its conjugates
lamCL=lamOL(1:4)*0.9;
% Measurement matrix
C=[eye(dof) zeros(dof)];
C=C(sen,:);
% Control force matrix
b2a=[0.9 0 0 0]';
b2b=[-0.9 0.9 0 0]';
b2=[b2a b2b];
B=[zeros(dof,2);inv(M)*b2];
% ************************************************
% SVD - pole placement
% ************************************************
m=length(sen);
for j=1:m
    V=null([Ac-eye(2*dof)*lamCL(j) -B]);
% We take the combining constants arbitrarily as [1;1];
alpha=[1;1];
T=V*alpha;
f(:,j)=T(1:2*dof);
b(:,j)=T(2*dof+1:end);
end;
CF=C*f;
G=b*inv(CF);
% We see that the gain is real (to eliminate the roundof)
G=real(G);
% G is quite large and thus possibly difficult to realize the
% associated forces
% Closed loop eigenstructure
ACL=Ac-B*G*C;
[aa,bb]=eig(ACL);
bb=diag(bb);
% We see that we've realized the poles but the eigenstructure is unstable
% ************************
% ************************
% We try a LS placement of all poles
lamCL=lamOL.*[0.9 0.9 0.9 0.9 1 1 1 1]';
for j=1:2*dof
    V=null([Ac-eye(2*dof)*lamCL(j) -B]);
% We take the combining constants arbitrarily;
alpha=[1;1];
T=V*alpha;
f(:,j)=T(1:2*dof);
b(:,j)=T(2*dof+1:end);
end;
CF=C*f;
GLS=b*pinv(CF);
% We see that the gain is real (to eliminate the roundof)
GLS=real(GLS);
% Note that G is quite large and thus possibly difficult to realize the
% associated forces
% Closed loop eigenstructure
ACL=Ac-B*GLS*C;
[aa,bbLS]=eig(ACL);
bbLS=diag(bbLS);
% *************************************************************************
% We see that now the gain is reasonable and the eigenstructure is stable
% *************************************************************************
% We compute the derivatives and compare with the OL
leftT=inv(aa);
Kp=zeros(4);
Kp(1,1)=1;
Ap=[zeros(dof) zeros(dof);
    -inv(M)*Kp zeros(dof)];
derCL=diag(leftT*Ap*aa);
% OL derivatives
[aa1,bb1]=eig(Ac);
leftT1=inv(aa1);
derOL=diag(leftT1*Ap*aa1);